﻿using System;
using System.Collections.Generic;
using System.DirectoryServices;
using System.DirectoryServices.AccountManagement;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Management;
using System.Text.RegularExpressions;
using System.Threading;
using System.ComponentModel; //BW
using System.Windows.Forms;
using Microsoft.ConfigurationManagement.ManagementProvider;
using Microsoft.ConfigurationManagement.ManagementProvider.WqlQueryEngine;
using Microsoft.Win32;
using System.Security.Cryptography;
using System.Text;


namespace SoftwareUpdateTool
{

    public partial class frmSUPTool : Form
    {
        public SynchronizationContext _syncContext = null;
        public bool abort = false;
        string Domain = Environment.UserDomainName;
        bool ShowWarning = false;
        
        public frmSUPTool()
        {
            InitializeComponent();
            txtDomain.Text = DNFqdn();
            this.dataGridView1.DefaultCellStyle.ForeColor = Color.Black;
            _syncContext = SynchronizationContext.Current;
            txtUser.Text = Environment.UserName;
            this.Text = "Toolio v." + Application.ProductVersion;

            // Set listbox month for updates
            DateTime date = DateTime.Now;
            int mo = date.Month;
            mo--; // Zero-based
            lstPatchMonth.SelectedIndex = mo;
            ShowWarning = true;

            if (getRegVal("DualEnvironment") == "1")

                dualEnvironmentToolStripMenuItem1.Checked = true;
            else
                dualEnvironmentToolStripMenuItem1.Checked = false;

            if (getRegVal("ServiceWindow") == "1")
                addMaintenanceWindowsToolStripMenuItem.Checked = true;
            else
                addMaintenanceWindowsToolStripMenuItem.Checked = false;

                 
        }

        private bool checkCreds()
        {
            bool test = true;

            if (txtDomain.Text == "") { test = false; MessageBox.Show("Domain name missing. Please enter domain.", "No Domain", MessageBoxButtons.OK, MessageBoxIcon.Exclamation); }
            if (cboSiteServer.Text == "") { test = false; MessageBox.Show("No site server. Please enter or select site server name.", "No Site Server", MessageBoxButtons.OK, MessageBoxIcon.Exclamation); }
            if (txtUser.Text == "") { test = false; MessageBox.Show("User name missing. Please enter user credentials.", "No User", MessageBoxButtons.OK, MessageBoxIcon.Exclamation); }
            if (txtPassword.Text == "") {test = false; MessageBox.Show("Password missing. Please enter password.", "No Password", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);}

            return test;
        }

        private void btnCheckScript_Click(object sender, EventArgs e)
        {
            if (checkCreds())
            {
                clearDGV();

                if (dataGridView1.Rows.Count != 0)
                    if (lstSUGs.SelectedIndex == -1)
                    {
                        MessageBox.Show("Please select a Software Update Group first.", "Software Update Group Not Selected", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        return;
                    }

                if (lstADGroups.Items.Count == 0) getADGroups(WMIConn());
                if (lstSUGs.Items.Count == 0) getSUGs(WMIConn());

                checkScript();
                dataGridView1.ClearSelection();

                updateTimeGrid();
            }
        }

        private void checkScript()
        {
            int i = 0;
            if (lstSUGs.SelectedIndex < 0)
            {
                MessageBox.Show("No software update groups selected.", "Select Software Update Group", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            //string SUGname = lstSUGs.SelectedItem.ToString();
            foreach (string SUGname1 in lstSUGs.SelectedItems)
            {
                if (lstADGroups.SelectedIndex < 0) // Will assume deploy to all
                {
                    foreach (string grp in lstADGroups.Items)
                    {
                        dataGridView1.Rows.Add();
                        dataGridView1.Rows[i].Cells[1].Value = grp;
                        dataGridView1.Rows[i].Cells[2].Value = SUGname1;
                        dataGridView1.Rows[i].Cells[0].Value = getCollectionName(grp);
                        i++;
                    }
                }
                
            }

            if (lstADGroups.SelectedIndex >= 0)
            {
                foreach(string SUGname1 in lstSUGs.SelectedItems)
                {
                    foreach (string grp in lstADGroups.SelectedItems)
                    {
                        dataGridView1.Rows.Add();
                        dataGridView1.Rows[i].Cells[1].Value = grp;
                        dataGridView1.Rows[i].Cells[2].Value = SUGname1;
                        dataGridView1.Rows[i].Cells[0].Value = getCollectionName(grp);
                        i++;
                    }
                }
            }
        }

        private void getADGroups(WqlConnectionManager connection)
        {
            lstADGroups.Items.Clear();
            string strRegex = @"^[a-z]{3}-SCCM\s[a-z]{3}-[0-9]{2}-[0-9]{4}-[0-9]{4}$";
            if (chkNoReboot.Checked) strRegex = @"^[a-z]{3}-SCCM\s[a-z]{3}-[0-9]{2}-NoReboot$";
            
            // Instantiate the regular expression object.
            Regex r = new Regex(strRegex, RegexOptions.IgnoreCase);
            if (txtDomain.Text == "")
            {
                MessageBox.Show("Please enter FQDN in domain box.", "No domain", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            string FQDN = txtDomain.Text;
            string OU;

            try
            {
                if (txtOU.Text != "")
                    OU = txtOU.Text + "," + GetLocalhostFqdn();
                else
                    OU = GetLocalhostFqdn();

                PrincipalContext yourOU = new PrincipalContext(ContextType.Domain, FQDN, OU);
                GroupPrincipal findAllGroups = new GroupPrincipal(yourOU, "*");
                PrincipalSearcher ps = new PrincipalSearcher(findAllGroups);
                foreach (var group in ps.FindAll())
                {
                    Match m = r.Match(group.SamAccountName);
                    if (m.Success)
                    {
                        lstADGroups.Items.Add(group.SamAccountName);
                    }
                }

                if (lstADGroups.Items.Count != 0)
                {
                    string exampleADGroup = lstADGroups.Items[0].ToString();

                    lblExceptionGrp.Text = getRegVal("EXCEPTION");
                }
                else
                {
                    warning();
                    if(chkNoReboot.Checked == true)
                        txtInfo.Text = "'No Reboot' groups not found.";
                    else
                        txtInfo.Text = "No software update groups found.";

                }
                
            }
            catch (Exception e)
            {
                warning();
                txtInfo.Text = e.Message;
            }

        }
        public void warning()
        {
            txtInfo.ForeColor = Color.Red;
            txtInfo.BackColor = Color.Black;
        }
        public void green()
        {
            txtInfo.ForeColor = Color.LightGreen;
            txtInfo.BackColor = Color.Black;
        }

        private List<string> checkGrid()
        {
            //  int cell = -100; //unlikely
            List<string> lst = new List<string>();

            for (int j = 0; j < dataGridView1.RowCount; j++)
            {
                for (int k = 0; k < 5; k++)
                {
                    lst.Add(dataGridView1.Rows[j].Cells[0].Value.ToString() + "," + dataGridView1.Rows[j].Cells[1].Value.ToString() + "," + dataGridView1.Rows[j].Cells[2].Value.ToString() + "," + dataGridView1.Rows[j].Cells[3].Value.ToString() + "," + dataGridView1.Rows[j].Cells[4].Value.ToString());
                }
            }
            // Remove duplicates
            lst = lst.Distinct().ToList();
            return lst;
        }

        private void btnRunScript_Click(object sender, EventArgs e)
        {
            if (checkCreds())
            {
                if (dataGridView1.Rows.Count == 0)
                {
                    MessageBox.Show("No schedule displayed. Run 'Get Schedule' first.", "Deploy", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }

                List<string> lst = new List<string>();
                lst = checkGrid();

                if (lst.Count == 0)
                {
                    var ques = MessageBox.Show("No updates selected in schedule. Do you wish to deploy all updates in the schedule?", "Deploy Updates", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if (ques == DialogResult.No) return;
                    dataGridView1.SelectAll();

                }

                progressBar1.Value = 0;
                dataGridView1.ClearSelection();
                var x = System.Threading.ThreadPool.QueueUserWorkItem(o =>
               {
                   if (configureUpdates(WMIConn()))
                   {
                       btnRunScript.Invoke(new MethodInvoker(delegate { btnRunScript.Text = "Deploy"; }));
                       MessageBox.Show("Updates Deployed Successfully.", "Update Status", MessageBoxButtons.OK, MessageBoxIcon.Information);
                   }
                   else
                   {
                       btnRunScript.Invoke(new MethodInvoker(delegate { btnRunScript.Text = "Deploy"; }));
                       MessageBox.Show("Updates Unsuccessful.", "Update Status", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                   }
                   progressBar1.Invoke(new MethodInvoker(delegate { progressBar1.Refresh(); }));

               }
               );

            }
        }

        private void clearDGV()
        {
            dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.Rows.Clear(); }));
            dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.DataSource = null; }));
            dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.Refresh(); }));
            dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.ClearSelection(); }));
        }

        private bool configureUpdates(WqlConnectionManager conn)
        {
            // See https://msdn.microsoft.com/en-us/library/hh949014.aspx

            if (txtUser.Text == "" || txtPassword.Text == "")
            {
                MessageBox.Show("Please enter username and password in appropriate box.");
                return false;
            }
            string lastPatchMonth = "";
            lstPatchMonth.Invoke(new MethodInvoker(delegate { lastPatchMonth = lstPatchMonth.Text; }));
            var q = MessageBox.Show("Please be aware you are setting up deployment dates for the " + lastPatchMonth + " schedule. You can set it up for previous months by selecting a different month in PATCH MONTH.", "Patch Month", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);
            if (q == DialogResult.Cancel) return false;

            string strADGrp = "";
            string strSUG = "";

            string strAssignmentName;
            int DesiredConfigType = 1;  // 1 (0x1)
            int DPLocality = 64; // 80 (0x50)
            int newLocaleId = 1033; // 1033 (0x409)
            bool newLogComplianceToWinEvent = false; //FALSE
            bool notifyUser = true; //TRUE
            bool raiseMOMAlertsOnFailure = false; //FALSE
            bool sendDetailedNonComplianceStatus = false; //FALSE
            int newSuppressReboot = 0; //0 (0x0)
            bool configUpdates = false;
            bool newORServiceWindows = false;
            bool WoL = true;
            bool newRebootServiceWindows = false;
            int assignmentType = 5;
            string newCollectionID;
            if (allowInstallRebbotOutsideMaintenanceWindowToolStripMenuItem.Checked)
            {
                newRebootServiceWindows = true;
                newORServiceWindows = true;
                var ans =MessageBox.Show("You have 'install and reboot outside maintenance windows' option selected. Is this what you want? If you select no, please go to the option menu and unselect as required.", "Option Check", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (ans == DialogResult.No) return false;
            }

            List<string> lst = new List<string>();
            lst = checkGrid();
            if (lst.Count > 0) // remove existing rows
            {

                clearDGV();
            }

            dataGridView1.ClearSelection();

            for (int i = 0; i < lst.Count; i++)
            {
                string[] arrEntry = lst.ElementAt(i).Split(',');

                dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.Rows.Add(); }));
                dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.Rows[i].Cells[0].Value = arrEntry[0]; }));
                dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.Rows[i].Cells[1].Value = arrEntry[1]; }));
                dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.Rows[i].Cells[2].Value = arrEntry[2]; }));
                dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.Rows[i].Cells[3].Value = arrEntry[3]; }));
                dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.Rows[i].Cells[4].Value = arrEntry[4]; }));

            }

            var ques = MessageBox.Show("You have selected to deploy to the schedule below. Is this what you want?", "Deploy Updates", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);
            if (ques == DialogResult.Cancel) return false;

            int UpdateCount = dataGridView1.RowCount;
            dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.ClearSelection(); }));
            btnRunScript.Invoke(new MethodInvoker(delegate { btnRunScript.Text = "Deploying Updates..."; }));
            progressBar1.Invoke(new MethodInvoker(delegate { progressBar1.Maximum = UpdateCount; }));
            string[] splADGroup;
            string endTime1 = "";
            string startTime1 = "";
            for (int i = 0; i < UpdateCount; i++)
            {
                string collectionName = dataGridView1.Rows[i].Cells[0].Value.ToString();
                strADGrp = dataGridView1.Rows[i].Cells[1].Value.ToString();
                splADGroup = strADGrp.Split('-');
                Int32 timeDuration = 0;
                if (splADGroup[3] != "NoReboot")
                {
                    endTime1 = splADGroup[4].Remove(2);
                    startTime1 = splADGroup[3].Remove(2);

                    Int32 ET = Convert.ToInt32(endTime1);
                    Int32 ST = Convert.ToInt32(startTime1);

                    if (ET < ST) ET += 24; // Does the maintenance window cross midnight?

                    timeDuration = ET - ST;
                }
                
                newCollectionID = GetCollectionID(conn, collectionName);
                if (newCollectionID == "")
                {
                    MessageBox.Show(collectionName + " can't be found. Please ensure the collection to be deployed to exists. Use the 'create collection' tool under the 'New' menu.", "Collection not found", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }
                int[] SUGIDs = getSoftwareUpdateGroupsFromCollection(conn, newCollectionID);
                strSUG = dataGridView1.Rows[i].Cells[2].Value.ToString();
                strAssignmentName = getAssignmentName(strSUG, strADGrp);
                string enforcementDeadline = getTime(strAssignmentName, true,false);
                string startTime = getTime(strAssignmentName, false,false);
                int[] CI_ID = getUpdatesFromUpdateGroup(conn, getCI(conn, strSUG));
                Int32 AssignedUpdateGroup = getCI(conn, strSUG);
                bool createAssignment = true;

                if (createAssignment)
                {
                    // Check that Software Update Group isn't already assigned to the collection
                    foreach (int SUGID in SUGIDs)
                    {
                        if (AssignedUpdateGroup == SUGID)
                        {
                            warning();
                            txtInfo.Invoke(new MethodInvoker(delegate { txtInfo.Text = collectionName + " already has " + strAssignmentName + " assigned."; }));
                            Log(collectionName + " already has " + strAssignmentName + " assigned.");
                            dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.Rows[i].DefaultCellStyle.BackColor = Color.GreenYellow; }));
                            createAssignment = false;
                        }
                    }
                }

                progressBar1.Invoke(new MethodInvoker(delegate { progressBar1.Value = i + 1; }));

                if (addMaintenanceWindowsToolStripMenuItem.Checked &! collectionName.Contains("No Reboot"))
                {
                    string MWstartTime = getTime(strAssignmentName, true, true);
                    string scheduleToken = CreateDailyRecurringScheduleToken(conn, timeDuration, 2, MWstartTime, true);
                    green();
                    txtInfo.Invoke(new MethodInvoker(delegate { txtInfo.Text = "Assigning maintenance window to " + collectionName + "\n"; }));
                    CreateMaintenanceWindow(conn, newCollectionID, collectionName, collectionName, scheduleToken, true, 1);
                }



                if (createAssignment) // Deploy Updates
                {
                    if (strADGrp.Contains("NoReboot"))
                    {
                        newRebootServiceWindows = false;
                        newORServiceWindows = false;
                        newSuppressReboot = 3;
                    }
                    configUpdates = ConfigureAndDeploySUMUpdates(conn, true, CI_ID, AssignedUpdateGroup, 2, strSUG, strAssignmentName, DesiredConfigType, DPLocality, newLocaleId, newLogComplianceToWinEvent, notifyUser, raiseMOMAlertsOnFailure, sendDetailedNonComplianceStatus, startTime, enforcementDeadline, newSuppressReboot, newCollectionID, true, newORServiceWindows, WoL, newRebootServiceWindows, assignmentType);

                    // Indicate if update successful by changing row colour
                    if (configUpdates)
                        dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.Rows[i].DefaultCellStyle.BackColor = Color.LightGreen; }));
                    else
                        dataGridView1.Invoke(new MethodInvoker(delegate { dataGridView1.Rows[i].DefaultCellStyle.BackColor = Color.Red; }));
                        
                }
            }

            return configUpdates;
        }


        public bool ConfigureAndDeploySUMUpdates(WqlConnectionManager connection,

                                               bool newApplyToSubTargets,
                                               int[] newArrayAssignedCIs,
                                               int AssignedUpdateGroup,
                                               int newAssignmentAction, // 1 = Detect, 2 = Apply
                                               string newAssignmentDescription,
                                               string newAssignmentName,
                                               int newDesiredConfigType, // 1 = required, 2 = not_allowed
                                               int newDPLocality, // DP_DOWNLOAD_FROM_LOCAL
                                               int newLocaleID,
                                               bool newLogComplianceToWinEvent,
                                               bool newNotifyUser,
                                               bool newRaiseMomAlertsOnFailure,
                                               bool newSendDetailedNonComplianceStatus,
                                               string newStartTime,
                                               string enforcementDeadline,
                                               int newSuppressReboot,
                                               string newTargetCollectionID,
                                               bool newUseGMTTimes,
                                               bool newORServiceWindows,
                                               bool WoL,
                                               bool newRebootServiceWindows,
                                               int assignmentType
                   )
        {
            try
            {
                // Create the deployment object.
                IResultObject newSUMUpdatesAssignment = connection.CreateInstance("SMS_UpdateGroupAssignment");

                // Populate new deployment properties.
                // Note: newTemplateName must be unique.

                newSUMUpdatesAssignment["ApplyToSubTargets"].BooleanValue = newApplyToSubTargets;
                newSUMUpdatesAssignment["AssignedCIs"].IntegerArrayValue = newArrayAssignedCIs;
                newSUMUpdatesAssignment["AssignedUpdateGroup"].IntegerValue = AssignedUpdateGroup;
                newSUMUpdatesAssignment["AssignmentAction"].IntegerValue = newAssignmentAction;
                newSUMUpdatesAssignment["AssignmentDescription"].StringValue = newAssignmentDescription;
                newSUMUpdatesAssignment["AssignmentName"].StringValue = newAssignmentName;
                newSUMUpdatesAssignment["DesiredConfigType"].IntegerValue = newDesiredConfigType;
                newSUMUpdatesAssignment["DPLocality"].IntegerValue = newDPLocality;
                newSUMUpdatesAssignment["LocaleID"].IntegerValue = newLocaleID;
                newSUMUpdatesAssignment["LogComplianceToWinEvent"].BooleanValue = newLogComplianceToWinEvent;
                newSUMUpdatesAssignment["NotifyUser"].BooleanValue = newNotifyUser;
                newSUMUpdatesAssignment["RaiseMomAlertsOnFailure"].BooleanValue = newRaiseMomAlertsOnFailure;
                newSUMUpdatesAssignment["NotifyUser"].BooleanValue = newNotifyUser;
                newSUMUpdatesAssignment["SendDetailedNonComplianceStatus"].BooleanValue = newSendDetailedNonComplianceStatus;
                newSUMUpdatesAssignment["StartTime"].StringValue = newStartTime;
                newSUMUpdatesAssignment["EnforcementDeadline"].StringValue = enforcementDeadline;
                newSUMUpdatesAssignment["SuppressReboot"].IntegerValue = newSuppressReboot;
                newSUMUpdatesAssignment["TargetCollectionID"].StringValue = newTargetCollectionID;
                newSUMUpdatesAssignment["UseGMTTimes"].BooleanValue = newUseGMTTimes;
                newSUMUpdatesAssignment["RebootOutsideOfServiceWindows"].BooleanValue = newRebootServiceWindows;
                newSUMUpdatesAssignment["OverrideServiceWindows"].BooleanValue = newORServiceWindows;
                newSUMUpdatesAssignment["WoLEnabled"].BooleanValue = WoL;
                newSUMUpdatesAssignment["assignmentType"].IntegerValue = assignmentType;
                newSUMUpdatesAssignment["PersistOnWriteFilterDevices"].BooleanValue = true;
                newSUMUpdatesAssignment["LimitStateMessageVerbosity"].BooleanValue = true;
                newSUMUpdatesAssignment["DisableMomAlerts"].BooleanValue = true;
                newSUMUpdatesAssignment["StateMessageVerbosity"].IntegerValue = 5;

                // Save new deployment and new deployment properties.
                newSUMUpdatesAssignment.Put();

                // Output the new deployment name.
                green();
                txtInfo.Invoke(new MethodInvoker(delegate { txtInfo.Text = "Created deployment: " + newAssignmentName; }));
                
                Log("Created deployment: " + newAssignmentName);
                return true;
            }

            catch (SmsException ex)
            {
                warning();
                txtInfo.Invoke(new MethodInvoker(delegate { txtInfo.Text = "Failed to create newSUMUpdatesAssignment. Error: " + ex.Message; }));
                
                Log("Failed to create newSUMUpdatesAssignment. Error: " + ex.Message);
                return false;
                throw;
            }
        }


        public int[] getSoftwareUpdateGroupsFromCollection(WqlConnectionManager connection, string CollectionID)
        {
            // SELECT AssignedUpdateGroup FROM SMS_UpdateGroupAssignment where TargetCollectionID='NPD000bb'
            try
            {
                string Query1 = "SELECT AssignedUpdateGroup FROM SMS_UpdateGroupAssignment where TargetCollectionID='" + CollectionID + "'";
                List<int> upd = new List<int>();
                IResultObject SUGAssignment = connection.QueryProcessor.ExecuteQuery(Query1);

                foreach (IResultObject AssignmentID in SUGAssignment)
                {
                    Dictionary<string, string> dictionary = new Dictionary<string, string>();
                    dictionary = AssignmentID.PropertyList;
                    string AssID;
                    dictionary.TryGetValue("AssignedUpdateGroup", out AssID);

                    upd.Add(Convert.ToInt32(AssID));

                }

                Int32[] arrUpdateGroups = upd.ToArray();

                return arrUpdateGroups;
            }
            catch (Exception e)
            {
                warning();
                txtInfo.Text = e.Message;
                Log(e.Message);
                return null;
            }
        }

        public int[] getUpdatesFromUpdateGroup(WqlConnectionManager connection, int SUGID)
        {
            try
            {
                string Query1 = "SELECT upd.* FROM SMS_SoftwareUpdate upd, SMS_CIRelation cr WHERE cr.FromCIID=" + SUGID + " AND cr.RelationType=1 AND upd.CI_ID=cr.ToCIID";
                List<int> upd = new List<int>();
                IResultObject ListOfUpdates = connection.QueryProcessor.ExecuteQuery(Query1);

                foreach (IResultObject update in ListOfUpdates)
                {
                    Dictionary<string, string> dictionary = new Dictionary<string, string>();
                    dictionary = update.PropertyList;
                    string CIID;
                    dictionary.TryGetValue("CI_ID", out CIID);

                    upd.Add(Convert.ToInt32(CIID));
                }

                Int32[] arrUpdates = upd.ToArray();
                return arrUpdates;
            }
            catch (SmsException ex)
            {
                warning();
                txtInfo.Text = "Failed to list the packages and programs. Error: " + ex.Message;
                Log("Failed to list the packages and programs. Error: " + ex.Message);
                throw;
            }
        }

        private string getNoRebootDate(string strAssignmentName)
        {
            string pSunday = patchSunday().Insert(4, "/");
            pSunday = pSunday.Insert(7, "/");
            DateTime dt = Convert.ToDateTime(pSunday);
            string[] splAssignmentName = strAssignmentName.Split(' ');
            string[] spldatetime = splAssignmentName[0].Split('_');
            switch (spldatetime[2])
            {
                case "WK1":
                    dt = dt.AddDays(0);
                    break;
                case "WK2":
                    dt = dt.AddDays(7);
                    break;
                case "WK3":
                    dt = dt.AddDays(14);
                    break;
                case "WK4":
                    dt = dt.AddDays(21);
                    break;

            }


            string[] dayNo = splAssignmentName[0].Split('_');

            

            dt = dt.AddDays(conDay(dayNo[3]));
            return dt.ToString();
        }

        private string getTime(string strAssignmentName, bool enforced, bool MW)
        {
            // eg 15/11/2015 00:00:00
            // eg "1015_MSUpdates_WK2_Friday (20151127_2300)"
            // return eg 20150923110200.000000+***
            try
            {

                string newdt = "";
                string startTime = "";
                string startDate = "";
                string[] splAssignmentName = strAssignmentName.Split(' ');
                string[] spldatetime = splAssignmentName[1].Split('_');
                if (strAssignmentName.Contains("No Reboot"))
                {
                    startDate = getNoRebootDate(strAssignmentName);
                    spldatetime = startDate.Split(' ');
                    newdt = spldatetime[0];
                    startTime = spldatetime[1];
                    if (enforced) startTime = "03:00:00";

                }
                else
                {
                    startDate = spldatetime[0].TrimStart('(');
                    startTime = spldatetime[1].TrimEnd(')');

                    startDate = startDate.Insert(4, "/");
                    startDate = startDate.Insert(7, "/");

                    DateTime stdt = Convert.ToDateTime(startDate);
                    newdt = stdt.ToShortDateString();

                    if (enforced == true)
                        startTime = startTime.Insert(2, ":");
                    else
                        startTime = "00:00:00";
                }

                string dmtfDateTime;

                if (MW == false)
                {
                    DateTime stDate = Convert.ToDateTime(newdt + " " + startTime).AddHours(1);
                    dmtfDateTime = ManagementDateTimeConverter.ToDmtfDateTime(stDate);
                }
                else
                {
                    DateTime stDate = Convert.ToDateTime(newdt + " " + startTime);
                    dmtfDateTime = ManagementDateTimeConverter.ToDmtfDateTime(stDate);
                }
                
                return dmtfDateTime;
            }
            catch
            {
                warning();
                txtInfo.Text = "Unable to obtain correct time entry. Have you selected a poorly-formatted deployment group?";
                return null;
            }
    
        }



        public bool CreateDynamicCollection(WqlConnectionManager connection, string newCollectionName, string newCollectionComment, bool ownedByThisSite, string query, string ruleName, string LimitToCollectionID, bool PatchesOnly, bool CreateCollections, bool CreateQueries)
        {

            try
            {
                // Create new SMS_Collection object.
                IResultObject newCollection = connection.CreateInstance("SMS_Collection");

                // Populate the new collection object properties.
                newCollection["Name"].StringValue = newCollectionName;
                newCollection["Comment"].StringValue = newCollectionComment;
                newCollection["OwnedByThisSite"].BooleanValue = ownedByThisSite;
                newCollection["LimitToCollectionID"].StringValue = LimitToCollectionID;
                newCollection["RefreshType"].IntegerValue = 4; // Incremental updates (CONTANT_UPDATE)

                // Save the new collection object and properties.
                // In this case, it seems necessary to 'get' the object again to access the properties.
                if (CreateCollections) newCollection.Put();
                newCollection.Get();

                // Validate the query.
                Dictionary<string, object> validateQueryParameters = new Dictionary<string, object>();
                validateQueryParameters.Add("WQLQuery", query);
                IResultObject result = connection.ExecuteMethod("SMS_CollectionRuleQuery", "ValidateQuery", validateQueryParameters);

                // Create query rule.
                IResultObject newQueryRule = connection.CreateInstance("SMS_CollectionRuleQuery");
                newQueryRule["QueryExpression"].StringValue = query;
                newQueryRule["RuleName"].StringValue = ruleName;

                // Add the rule. Although not used in this sample, QueryID contains the query identifier.
                Dictionary<string, object> addMembershipRuleParameters = new Dictionary<string, object>();
                addMembershipRuleParameters.Add("collectionRule", newQueryRule);
                IResultObject queryID = newCollection.ExecuteMethod("AddMembershipRule", addMembershipRuleParameters);

                // Start collection evaluator.
                newCollection.ExecuteMethod("RequestRefresh", null);
                green();
                txtInfo.Invoke(new MethodInvoker(delegate { txtInfo.Text = "Created collection: " + newCollectionName; })); 
                Log("Created collection: " + newCollectionName);
                return true;
            }
            catch (SmsException ex)
            {
                warning();
                txtInfo.Invoke(new MethodInvoker(delegate { txtInfo.Text = "Failed to create collection. Error: " + ex.Message; }));
                Log("Failed to create collection. Error: " + ex.Message);
                return false;
                throw;

            }
        }


        private string getAssignmentName(string SUG, string ADGrp)
        {
            
            string collection = getCollectionName(ADGrp).Replace("Software Updates - ", "");
            collection = collection.Replace(" ", "");
            string[] splName = collection.Split('-');
            string pDay = patchDay(ADGrp);
            string assignmentName = "";
            if (ADGrp.Contains("NoReboot"))
                assignmentName = SUG + "_" + splName[0] + "_" + splName[1] + " (No Reboot)";
            else
            {

                if (splName[0] == "WK3" || splName[0] == "WK4")
                {
                    pDay = pDay.Insert(4, "/");
                    pDay = pDay.Insert(7, "/");
                    DateTime dt = Convert.ToDateTime(pDay);
                    dt = dt.AddDays(14);

                    string[] splDT = String.Format("{0:yyyyMMdd}", dt).Split(' ');
                    pDay = splDT[0].Replace("/", "");

                }

                assignmentName = SUG + "_" + splName[0] + "_" + splName[1] + " (" + pDay + "_" + splName[2] + ")";
            }
            return assignmentName;
        }



        private string GetCollectionID(WqlConnectionManager connection, string CollectionName)
        {
            try
            {
                string Query1 = "SELECT * FROM SMS_Collection WHERE Name = '" + CollectionName + "'";
                string result = "";

                IResultObject ListOfResources1 = connection.QueryProcessor.ExecuteQuery(Query1);

                foreach (IResultObject Resource1 in ListOfResources1)
                {
                    result = Resource1["CollectionID"].StringValue;
                }

                return result;
            }
            catch (SmsException ex)
            {
                warning();
                txtInfo.Text = "Failed to list the packages and programs. Error: " + ex.Message;
                Log("Failed to list the packages and programs. Error: " + ex.Message);
                throw;
            }
        }

        private void getSUGs(WqlConnectionManager connection)
        {
            try
            {
                string strScope = connection.NamedValueDictionary["Connection"].ToString();
                ManagementScope scope = new ManagementScope(strScope);
                WqlObjectQuery wqlQuery = new WqlObjectQuery(string.Format("select * from SMS_AuthorizationList"));
                ManagementObjectSearcher searcher = new ManagementObjectSearcher(scope, wqlQuery);
                List<string> upd = new List<string>();

                string strRegex = @"^.*_MSUpdates$";
                Regex r = new Regex(strRegex, RegexOptions.IgnoreCase);

                foreach (ManagementObject CI in searcher.Get())
                {
                    string SUGName = CI.GetPropertyValue("LocalizedDisplayName").ToString();
                    Match m = r.Match(SUGName);
                    if (m.Success) lstSUGs.Items.Add(SUGName);
                }
            }
            catch 
            {
                MessageBox.Show("Unable to display Software Update Groups. Please ensure your credentials correct, you have selected the correct primary site server and you are not behind a firewall.", "Unable to access SUGs", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

        }

        private int getCI(WqlConnectionManager connection, string SUG)
        {

            string updatesGroup = SUG;
            string strScope = connection.NamedValueDictionary["Connection"].ToString();
            ManagementScope scope = new ManagementScope(strScope);
            WqlObjectQuery wqlQuery = new WqlObjectQuery(string.Format("select CI_ID from SMS_AuthorizationList Where LocalizedDisplayName = '{0}'", updatesGroup));
            ManagementObjectSearcher searcher = new ManagementObjectSearcher(scope, wqlQuery);

            string ret = "";
            foreach (ManagementObject CI in searcher.Get())
            {
                ret += CI.ToString();
            }

            string[] retspl = ret.Split('=');
            return Convert.ToInt32(retspl[1]);

        }

        public WqlConnectionManager WMIConn()
        {

            string computer = "";
            string user = "";
            string password = "";

            cboSiteServer.Invoke(new MethodInvoker(delegate { computer = cboSiteServer.Text; }));
            txtUser.Invoke(new MethodInvoker(delegate { user = txtUser.Text; }));
            txtPassword.Invoke(new MethodInvoker(delegate { password = txtPassword.Text; }));

            WqlConnectionManager WMIConnection = Connect(computer, Environment.UserDomainName + "\\" + user, password);
            return WMIConnection;

        }

        public WqlConnectionManager Connect(string serverName, string userName, string userPassword)
        {
            try
            {
                SmsNamedValuesDictionary namedValues = new SmsNamedValuesDictionary();
                WqlConnectionManager connection = new WqlConnectionManager(namedValues);

                if (System.Net.Dns.GetHostName().ToUpper() == serverName.ToUpper())
                {
                    // Connect to local computer.
                    connection.Connect(serverName);
                }
                else
                {
                    // Connect to remote computer.
                    connection.Connect(serverName, userName, userPassword);
                    
                }

                txtConnInfo.ForeColor = Color.LightGreen;
                txtConnInfo.Text = "Connected successfully to " + serverName;
                return connection;
            }
            catch (SmsException e)
            {
                txtConnInfo.ForeColor = Color.Red;
                txtConnInfo.Text = "Failed to Connect. Error: " + e.Message;
                return null;
            }
            catch (UnauthorizedAccessException e)
            {
                warning();
                txtConnInfo.Text = "Failed to authenticate. Error:" + e.Message;
                return null;
            }
        }


        private void cboSiteServer_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                WqlConnectionManager connection = WMIConn();

                string serverFQDN = cboSiteServer.Text + "." + txtDomain.Text;

                IResultObject siteCode = connection.QueryProcessor.ExecuteQuery("SELECT * FROM SMS_Site Where ServerName = '" + serverFQDN + "'");

                //MessageBox.Show(siteCode["SiteCode"].ToString());
                // Enumerate the allCollectionSettings collection (there should be just one item) and save the instance.
                foreach (IResultObject code in siteCode)
                {
                    txtSiteCode.Text = code["SiteCode"].StringValue;
                }

                setRegValue("SITECODE", txtSiteCode.Text);
                setRegValue("SITESERVER", cboSiteServer.Text);
            }
            catch
            {
                warning();
                txtInfo.Text = "Ensure that the site server and domain name is correct and contactable.";
            }
        }

        private string patchSunday() // First Sunday after patch Tuesday
        {
            DateTime date = DateTime.Now;
            int patchMonth = 0;
            lstPatchMonth.Invoke(new MethodInvoker(delegate { patchMonth = Convert.ToDateTime(lstPatchMonth.SelectedItem.ToString() + " 01, 1900").Month; }));
            int yr = date.Year;
            int mo = date.Month;
            if (patchMonth > mo) yr--;
            if (mo != patchMonth) mo = patchMonth;
            DateTime _date = new DateTime(yr, mo, 12);
            DateTime PatchSunday = _date.AddDays(2 - Convert.ToInt32(_date.DayOfWeek) + 5); // Patch Tuesday + 5 days
            
            return PatchSunday.ToString("yyyyMMd");
        }

        private string patchDay(string ADGrp)
        {
            string pSunday = patchSunday().Insert(4, "/");
            pSunday = pSunday.Insert(7, "/");
            string[] splGrp = ADGrp.Split('-');
            int pDay = Convert.ToInt32(splGrp[2]);

            DateTime dt = Convert.ToDateTime(pSunday);

            if (dualEnvironmentToolStripMenuItem1.Checked)
            {
                if (ADGrp.Contains("NPD")) dt.AddDays(pDay);
                if (ADGrp.Contains("PRD")) dt.AddDays(14 + pDay);
            }
            else
            {
                if (ADGrp.Contains("PRD")) dt.AddDays(pDay);
            }

            DateTime newDate = dt.AddDays(pDay - 1);

            return newDate.ToString("yyyyMMdd");
        }

        private string getCollectionName(string ADGrp)
        {
            if (ADGrp == "")
            {
                MessageBox.Show("No AD group selected. Please select an AD group.", "Select AD group", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return null;
            }
            string[] splGrp = ADGrp.Split('-');
            int pDay = Convert.ToInt32(splGrp[2]);
            string pSunday = patchSunday().Insert(4, "/");
            pSunday = pSunday.Insert(7, "/");
            DateTime dt = Convert.ToDateTime(pSunday);
            string timeRange = "";
            //Get Collection Name From Group
            //If production then has to be WK3 or WK4
            string strStart = "Software Updates - ";

            if (ADGrp.Contains("NoReboot"))
                timeRange = "(No Reboot)";
            else
                timeRange = splGrp[3] + "-" + splGrp[4];

            string wkDay = dt.AddDays(pDay - 1).ToString("dddd");
            string wk = "WK1";

            if (dualEnvironmentToolStripMenuItem1.Checked)
            {
                if (ADGrp.Contains("NPD")) // Acceptancee
                {
                    if (pDay <= 7)
                        wk = "WK1";
                    else
                        wk = "WK2";
                }
                else // Production
                {
                    if (pDay <= 7)
                        wk = "WK3";
                    else
                        wk = "WK4";
                }
            }
            else
            {
                if (ADGrp.Contains("PRD")) // Acceptancee
                {
                    if (pDay <= 7)
                        wk = "WK1";
                    else
                        wk = "WK2";
                }
            }

            string collectionName = strStart + wk + " - " + wkDay + " - " + timeRange; // eg 'Software Updates - WK3 - Sunday - 1900-2300'

            return collectionName;
        }



        private string DNFqdn()
        {
            var ipProperties = System.Net.NetworkInformation.IPGlobalProperties.GetIPGlobalProperties();
            return ipProperties.DomainName.ToString();
        }

        private string GetLocalhostFqdn()
        {
            var ipProperties = System.Net.NetworkInformation.IPGlobalProperties.GetIPGlobalProperties();
            string fqdn = ipProperties.DomainName.ToString();
            string[] spl = fqdn.Split('.');
            string dc = ",DC=";
            for (int i = 0; i < spl.Length; i++)
            {
                fqdn += dc + spl[i];
            }

            fqdn = fqdn.Replace(txtDomain.Text, "").TrimStart(',');

            return fqdn;
        }

        private void btnFindGroups_Click(object sender, EventArgs e)
        {
            if(checkCreds())
            {
                btnFindGroups.Text = "Searching...";
                getADGroups(WMIConn());
                btnFindGroups.Text = "Find Maintenance Window Groups";
            }
        }

        private void btnFindSUGs_Click(object sender, EventArgs e)
        {
            if (checkCreds())
            {
                btnFindSUGs.Text = "Searching...";
                getSUGs(WMIConn());
                btnFindSUGs.Text = "Find Software Update Groups";
            }
        }


        private void Log(string text)
        {
            DateTime date = DateTime.Now;
            var shortDate = date.ToString("yyMMdd");

            using (StreamWriter writer = new StreamWriter("C:\\Temp\\" + shortDate + "_SoftwareUpdateTool.log", true))
            {
                writer.WriteLine(text + "\n\n");
            }

        }

        private void btnOpenLogFile_Click(object sender, EventArgs e)
        {

            string textLine = "";
            DateTime date = DateTime.Now;
            var shortDate = date.ToString("yyMMdd");
            // Read the file and display it line by line.
            if (File.Exists("C:\\Temp\\" + shortDate + "_SoftwareUpdateTool.log"))
            {
                StreamReader objReader = new StreamReader("C:\\Temp\\" + shortDate + "_SoftwareUpdateTool.log");
                do
                {

                    textLine += objReader.ReadLine() + "\r\n";
                    lstADGroups.Items.Add(textLine);

                } while (objReader.Peek() != -1);

            }
            else
                return;

        }
        

        private string DetermineHoveredItem()
        {
            Point screenPosition = ListBox.MousePosition;
            Point listBoxClientAreaPosition = lstADGroups.PointToClient(screenPosition);

            int hoveredIndex = lstADGroups.IndexFromPoint(listBoxClientAreaPosition);
            if (hoveredIndex != -1)
                return lstADGroups.Items[hoveredIndex] as string;
            else
                return null;
        }

        private void btnCreateADGroups_Click(object sender, EventArgs e)
        {

        }

        private void createADGroups()
        {
            abort = false;
            frmGroupTemplate ft = new frmGroupTemplate(this);
            ft.StartPosition = FormStartPosition.CenterParent;
            ft.ShowDialog(this);

            if (!abort == true)
            {
                if (getRegValues() != null)
                {
                    Dictionary<string, string> templateValues = getRegValues();
                    String value;
                    bool valST2 = false;
                    bool valST3 = false;
                    bool exc = false;
                    string nrb = "";
                    if (templateValues.TryGetValue("ST2", out value)) valST2 = true;
                    if (templateValues.TryGetValue("ST3", out value)) valST3 = true;
                    if (templateValues.TryGetValue("EXCEPTION", out value)) exc = true;

                    for (int j = 1; j < 15; j++) // Day loop
                    {
                        string t;
                        string newADGroupName = "";

                        // double digits
                        if (j < 10)
                            t = "0" + j.ToString();
                        else
                            t = j.ToString();

                        newADGroupName = templateValues["SITE"] + "-SCCM " + templateValues["ENVIRONMENT"] + "-" + t + "-" + templateValues["ST1"] + "-" + templateValues["ET1"];
                        try
                        {
                            if (CreateGroup(txtOU.Text + "," + GetLocalhostFqdn(), newADGroupName))
                            {
                                green();
                                txtInfo.Text = newADGroupName + " Created.\r\n";
                                txtInfo.Refresh();
                                Log(newADGroupName + " Created.");
                            }
                        }
                        catch (Exception ex)
                        {
                            warning();
                            txtInfo.Text = newADGroupName + " failed. \r\n" + ex.Message;
                            txtInfo.Refresh();
                            Log(newADGroupName + ": " + ex.Message);
                        }


                        if (valST2)
                        {
                            newADGroupName = templateValues["SITE"] + "-SCCM " + templateValues["ENVIRONMENT"] + "-" + t + "-" + templateValues["ST2"] + "-" + templateValues["ET2"];
                            try
                            {
                                if (CreateGroup(txtOU.Text + "," + GetLocalhostFqdn(), newADGroupName))
                                {
                                    green();
                                    txtInfo.Text = newADGroupName + " Created.\r\n";
                                    txtInfo.Refresh();
                                    Log(newADGroupName + " Created.");
                                }
                            }
                            catch (Exception ex)
                            {
                                warning();
                                txtInfo.Text = newADGroupName + " failed. \r\n" + ex.Message;
                                txtInfo.Refresh();
                                Log(newADGroupName + ": " + ex.Message);
                            }
                        }

                        

                        if (valST3)
                        {
                            newADGroupName = templateValues["SITE"] + "-SCCM " + templateValues["ENVIRONMENT"] + "-" + t + "-" + templateValues["ST3"] + "-" + templateValues["ET3"];
                            try
                            {
                                if (CreateGroup(txtOU.Text + "," + GetLocalhostFqdn(), newADGroupName))
                                {
                                    green();
                                    txtInfo.Text = newADGroupName + " Created.\r\n";
                                    txtInfo.Refresh();
                                    Log(newADGroupName + " Created.\r\n");
                                }
                            }
                            catch (Exception ex)
                            {
                                warning();
                                txtInfo.Text = newADGroupName + " failed. " + ex.Message;
                                Log(newADGroupName + ": " + ex.Message);
                            }
                        }


                        // No Reboot Group
                        if(nrb!=t)
                            newADGroupName = templateValues["SITE"] + "-SCCM " + templateValues["ENVIRONMENT"] + "-" + t + "-NoReboot";
                            nrb = t;
                            try
                            {
                                if (CreateGroup(txtOU.Text + "," + GetLocalhostFqdn(), newADGroupName))
                                {
                                    green();
                                    txtInfo.Text = newADGroupName + " Created.\r\n";
                                    txtInfo.Refresh();
                                    Log(newADGroupName + " Created.\r\n");
                                }
                            }
                            catch (Exception ex)
                            {
                                warning();
                                txtInfo.Text = newADGroupName + " failed. " + ex.Message;
                                Log(newADGroupName + ": " + ex.Message);
                            }



                        if (exc)
                        {
                            newADGroupName = templateValues["EXCEPTION"];
                            try
                            {
                                if (CreateGroup(txtOU.Text + "," + GetLocalhostFqdn(), newADGroupName))
                                {
                                    green();
                                    txtInfo.Text = newADGroupName + " Created.\r\n";
                                    txtInfo.Refresh();
                                    Log(newADGroupName + " Created.\r\n");
                                }
                            }
                            catch (Exception ex)
                            {
                                warning();
                                txtInfo.Text = newADGroupName + " failed. " + ex.Message;
                                Log(newADGroupName + ": " + ex.Message);
                            }
                        }

                    }
                }
            }
        }

        private void setRegValue(string name, string value)
        {
            RegistryKey regKey = Registry.CurrentUser.CreateSubKey("SOFTWARE\\Bondytech\\SoftwareUpdateTool");
            regKey.SetValue(name, value);
        }

        private Dictionary<string, string> getRegValues()
        {
            try
            {
                string key = @"HKEY_CURRENT_USER\Software\Bondytech\SoftwareUpdateTool";
                string SITE = Registry.GetValue(key, "SITE", null).ToString();
                string ENVIRONMENT = Registry.GetValue(key, "ENVIRONMENT", null).ToString();
                string ST1 = Registry.GetValue(key, "ST1", null).ToString();
                string ET1 = Registry.GetValue(key, "ET1", null).ToString();
                string ST2 = Registry.GetValue(key, "ST2", null).ToString();
                string ET2 = Registry.GetValue(key, "ET2", null).ToString();
                string ST3 = Registry.GetValue(key, "ST3", null).ToString();
                string ET3 = Registry.GetValue(key, "ET3", null).ToString();
                string EXC = Registry.GetValue(key, "EXCEPTION", null).ToString();

                Dictionary<string, string> regVals = new Dictionary<string, string>();
                regVals.Add("SITE", SITE);
                regVals.Add("ENVIRONMENT", ENVIRONMENT);
                regVals.Add("ST1", ST1);
                regVals.Add("ET1", ET1);
                regVals.Add("ST2", ST2);
                regVals.Add("ET2", ET2);
                regVals.Add("ST3", ST3);
                regVals.Add("ET3", ET3);
                regVals.Add("EXCEPTION", EXC);

                return regVals;
            }
            catch
            {
                warning();
                txtInfo.Text = "AD group template not created.";
                return null;
            }
        }


        public bool CreateGroup(string ouPath, string name)
        {
            if (txtOU.Text == "")
            {
                MessageBox.Show("Please populate OU field, eg 'OU=SCCM Groups'");
                return false;
            }
            ouPath = ouPath.TrimStart(',');
            if (!DirectoryEntry.Exists("LDAP://CN=" + name + "," + ouPath))
            {
                try
                {
                    string strDomain = txtDomain.Text;
                    strDomain = strDomain.Split('.')[0];
                    string strUser = txtUser.Text;
                    string strPassword = txtPassword.Text;

                    // bind to the container, e.g. LDAP://cn=Users,dc=...
                    DirectoryEntry entry = new DirectoryEntry(@"LDAP://" + ouPath, strDomain + "\\" + strUser, strPassword, AuthenticationTypes.None);
 
                    // create group entry
                    DirectoryEntry group = entry.Children.Add("CN=" + name, "group");

                    // set properties
                    group.Properties["sAmAccountName"].Value = name;

                    if(!name.Contains("--")) // This indicates that the group isn't fully contructed meaning there is less than three windows
                    {
                        // save group
                        group.CommitChanges();
                        green();
                        txtInfo.Text = name + " created successfully.";
                        return true;
                    }
                    if(name.Contains("--")) return true; //expected in certain scenarios
                    return false; //otherwise
                }
                catch (Exception e)
                {
                    warning();
                    txtInfo.Text = e.Message.ToString();
                    return false;
                }
            }
            else
            {
                warning();
                txtInfo.Text = "Group already exists";
                return false;
            }
        }

        private void aDGroupsToolStripMenuItem_Click(object sender, EventArgs e)
        {

                createADGroups(); ;  // wait for a while
                Cursor.Current = Cursors.Default;

        }

        private void collectionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            btnFindGroups.Text = "Searching...";
            getADGroups(WMIConn());
            btnFindGroups.Text = "Find Maintenance Window Groups";

            List<string> lst = new List<string>();
            for (int i = 0; i < lstADGroups.Items.Count; i++)
            {
                lst.Add(lstADGroups.Items[i].ToString());
            }

            string msg = "";
            if (dualEnvironmentToolStripMenuItem1.Checked)
            {
                msg = "You are about to create collections on " + cboSiteServer.Text + " based on all the AD groups under 'Maintenance Windows'. This will automatically associate a query with each collection tying them to the appropriate AD group.\nYou also have 'Dual Environment' checked. If these collections are to be created for the PRD environment, this will create collections starting from week 3. If NPD, then they will be created from week 1. For more information see help menu.\n\n Go ahead?";
            }
            else
            {
                msg = "You are about to create collections on " + cboSiteServer.Text + " based on all the AD groups under 'Maintenance Windows'. This will automatically associate a query with each collection tying them to the appropriate AD group.\n\n Go ahead?";
            }
            
            var q = MessageBox.Show(msg, "Create Collections",MessageBoxButtons.OKCancel, MessageBoxIcon.Question);
            if (q == DialogResult.Cancel) return;

            var x = System.Threading.ThreadPool.QueueUserWorkItem(o =>
               {
                   createCollections(WMIConn(), lst);
               });
            Cursor.Current = Cursors.Default;
            progressBar1.Value = 0;
        }


        public void createCollections(WqlConnectionManager conn, List<string> lst)
        {
            if (lst.Count == 0)
            {
                MessageBox.Show("No maintenance windows groups displayed. If none are listed then these must first be created using NEW|AD Groups menu. These are required for creation of associated queries.", "No AD Groups",MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
            string strQuery = "";
            string strQueryName = "";
            string strExceptionGroup = "";
            string strCollName = "";
            bool createCollections = true;
            bool createMembershipRules = true;
            bool createAssignment = true;
            string strADGrp = "";
            string Week = "";
            strExceptionGroup = lblExceptionGrp.Text;
            int j = 1;
            string dayNoOld = "";
            progressBar1.Invoke(new MethodInvoker(delegate { progressBar1.Maximum = lst.Count; }));
            progressBar1.Invoke(new MethodInvoker(delegate { progressBar1.Value = 0; }));
            

                for (int i = 0; i < lst.Count; i++)
                {
                    string grp = lst[i];

                    string[] splGrp = grp.Split('-');
                    string dayno = splGrp[2];
                    string stTime = "";
                    string endTime = "";


                    if (grp.Contains("NoReboot"))
                    {
                        
                        // Do nothing
                    }
                    else
                    {
                        stTime = splGrp[3];
                        endTime = splGrp[4];
                    }
                    string day = "Sunday";
                    string DescDayNo = dayno;
                    if (dualEnvironmentToolStripMenuItem1.Checked)
                    {
                        if (grp.Contains("NPD")) Week = "1";
                        if (grp.Contains("PRD")) Week = "3";
                    }
                    else
                    {
                        if (grp.Contains("PRD") || grp.Contains("NPD")) Week = "1";
                    }
                    strADGrp = grp;
                    

                    if (dayno == dayNoOld) j--; // For days with more than one time

                    int dn = Convert.ToInt32(dayno);
                    dn--; // 0 indexed so -1
                    j = dn +1;
                    if (dn > 6) dn = dn - 7;
                    day = ((DayOfWeek)dn).ToString();
                    if (Week == "1") // Acceptancee
                    {
                        if (j <= 7)
                            Week = "WK1";
                        else
                            Week = "WK2";
                    }
                    else // Production
                    {
                        if (j <= 7)
                            Week = "WK3";
                        else
                            Week = "WK4";
                    }
                    if (!grp.Contains("NoReboot"))
                    {
                        strCollName = "Software Updates - " + Week + " - " + day + " - " + stTime + "-" + endTime;
                        strQueryName = Week + " - " + day + " - " + stTime + "-" + endTime;
                    }
                    else
                    {
                        strCollName = "Software Updates - " + Week + " - " + day + " - " + "(No Reboot)";
                        strQueryName = Week + " - " + day + " - " + "(No Reboot)";
                    }



                    string collectionExists = GetCollectionID(conn, strCollName);
                    string Dom = Environment.UserDomainName;
                    if (collectionExists == "")
                    {
                        strQuery = "select SMS_R_SYSTEM.ResourceID,SMS_R_SYSTEM.ResourceType,SMS_R_SYSTEM.Name,SMS_R_SYSTEM.SMSUniqueIdentifier,SMS_R_SYSTEM.ResourceDomainORWorkgroup,SMS_R_SYSTEM.Client from SMS_R_System where SMS_R_System.SystemGroupName = '" + Dom + @"\\" + strADGrp + "' and SMS_R_System.Name not in (select distinct SMS_R_System.Name from  SMS_R_System where SMS_R_System.SystemGroupName = '" + Dom + @"\\" + strExceptionGroup + "')";
                        CreateDynamicCollection(conn, strCollName, strQueryName + ".\n" + grp, true, strQuery, strQueryName, "SMS00001", createAssignment, createCollections, createMembershipRules);
                        txtInfo.Invoke(new MethodInvoker(delegate {
                            txtInfo.Text = "Creating " + strCollName + "...";
                            txtInfo.Refresh();
                        }));
                    }
                    else
                    {
                        warning();
                        txtInfo.Invoke(new MethodInvoker(delegate { txtInfo.Text = strCollName + " already exists, skipping."; }));
                    }

                    progressBar1.Invoke(new MethodInvoker(delegate { progressBar1.Value += 1; }));
                    dayNoOld = dayno;
                    j++;
                }




        }

        private void frmSUPTool_Load(object sender, EventArgs e)
        {
            newToolStripMenuItem.ForeColor = Color.White;
            helpToolStripMenuItem.ForeColor = Color.White;
            optionsToolStripMenuItem.ForeColor = Color.White;
            cboSiteServer.Text = getRegVal("SITESERVER");
            txtSiteCode.Text = getRegVal("SITECODE");
            string ou = getRegVal("StartOU");

            if (getRegVal("Password") != "")
            {
                String KeyString = GenerateAPassKey("PassKey");
                string enc = Decrypt(getRegVal("Password"), KeyString);
                txtPassword.Text = enc;
            }

            if (ou != "") txtOU.Text = ou;
            
        }

        private string getRegVal(string name)
        {
            RegistryKey regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Bondytech\\SoftwareUpdateTool");
            try
            {
                string ret = regKey.GetValue(name).ToString();
                if (ret == null)
                    return "";
                else
                    return ret;
            }
            catch (NullReferenceException)
            {
                setRegValue(name, "");
                return "";
            }
        }

        private void txtOU_TextChanged(object sender, EventArgs e)
        {
            setRegValue("StartOU", txtOU.Text);
        }

        private void txtPassword_TextChanged(object sender, EventArgs e)
        {
            if (chkSavePW.Checked)
            {
                String KeyString = GenerateAPassKey("PassKey");
                string enc = Encrypt(txtPassword.Text, KeyString);
                setRegValue("Password", enc);
            }
        }

        private void chkSavePW_CheckedChanged(object sender, EventArgs e)
        {
            if (chkSavePW.Checked)
            {
                String KeyString = GenerateAPassKey("PassKey");
                string enc = Encrypt(txtPassword.Text, KeyString);
                setRegValue("Password", enc);
                
            }
        }


        private int conDay (string day)
        {
            switch (day)
            {
                case "Sunday": return 0;
                case "Monday": return 1;
                case "Tuesday": return 2;
                case "Wednesday": return 3;
                case "Thursday": return 4;
                case "Friday": return 5;
                case "Saturday": return 6;
            }
            return -1;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            string test = "20151123";
            test = test.Insert(4, "/");
            test = test.Insert(7, "/");
            DateTime dt = Convert.ToDateTime(test);
            dt = dt.AddDays(14);
            
            MessageBox.Show(dt.ToString());
        }



        private void lstPatchMonth_SelectedValueChanged(object sender, EventArgs e)
        {
            if(ShowWarning)
            MessageBox.Show("By changing this value, you are effectively saying that you want all patch dates based of the " + lstPatchMonth.Text.ToUpper() + " patching schedule. Potentially this could automatically deploy these updates to any current collection members that aren't compliant. Consider whether this is what you want. Typically you might want to do this if you are in a new month and wish to apply the current (previous) month's schedule to your collections.", "Change Patching Scedule", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            
        }

        private void btnDeselect_Click(object sender, EventArgs e)
        {
            lstADGroups.SelectedItems.Clear();
        }


        public static string GenerateAPassKey(string passphrase)
        {
            // Pass Phrase can be any string
            string passPhrase = passphrase;
            // Salt Value can be any string(for simplicity use the same value as used for the pass phrase)
            string saltValue = passphrase;
            // Hash Algorithm can be "SHA1 or MD5"
            string hashAlgorithm = "SHA1";
            // Password Iterations can be any number
            int passwordIterations = 2;
            // Key Size can be 128,192 or 256
            int keySize = 256;
            // Convert Salt passphrase string to a Byte Array
            byte[] saltValueBytes = Encoding.ASCII.GetBytes(saltValue);
            // Using System.Security.Cryptography.PasswordDeriveBytes to create the Key
            PasswordDeriveBytes pdb = new PasswordDeriveBytes(passPhrase, saltValueBytes, hashAlgorithm, passwordIterations);
            //When creating a Key Byte array from the base64 string the Key must have 32 dimensions.
            byte[] Key = pdb.GetBytes(keySize / 11);
            String KeyString = Convert.ToBase64String(Key);

            return KeyString;
        }

        //Save the keystring some place like your database and use it to decrypt and encrypt
        //any text string or text file etc. Make sure you dont lose it though.

        public static string Encrypt(string plainStr, string KeyString)
        {
            RijndaelManaged aesEncryption = new RijndaelManaged();
            aesEncryption.KeySize = 256;
            aesEncryption.BlockSize = 128;
            aesEncryption.Mode = CipherMode.ECB;
            aesEncryption.Padding = PaddingMode.ISO10126;
            byte[] KeyInBytes = Encoding.UTF8.GetBytes(KeyString);
            aesEncryption.Key = KeyInBytes;
            byte[] plainText = ASCIIEncoding.UTF8.GetBytes(plainStr);
            ICryptoTransform crypto = aesEncryption.CreateEncryptor();
            byte[] cipherText = crypto.TransformFinalBlock(plainText, 0, plainText.Length);
            return Convert.ToBase64String(cipherText);
        }

        public static string Decrypt(string encryptedText, string KeyString)
        {
            RijndaelManaged aesEncryption = new RijndaelManaged();
            aesEncryption.KeySize = 256;
            aesEncryption.BlockSize = 128;
            aesEncryption.Mode = CipherMode.ECB;
            aesEncryption.Padding = PaddingMode.ISO10126;
            byte[] KeyInBytes = Encoding.UTF8.GetBytes(KeyString);
            aesEncryption.Key = KeyInBytes;
            ICryptoTransform decrypto = aesEncryption.CreateDecryptor();
            byte[] encryptedBytes = Convert.FromBase64CharArray(encryptedText.ToCharArray(), 0, encryptedText.Length);
            return ASCIIEncoding.UTF8.GetString(decrypto.TransformFinalBlock(encryptedBytes, 0, encryptedBytes.Length));
        }



        private void updateTimeGrid()
        {
            for (int i = 0; i < dataGridView1.Rows.Count; i++)
            {
                string strADGrp = dataGridView1.Rows[i].Cells[1].Value.ToString();
                string strSUG = dataGridView1.Rows[i].Cells[2].Value.ToString();
                string strAssignmentName = getAssignmentName(strSUG, strADGrp);
                string enforcementDeadline = getTime(strAssignmentName, true,false);
                string startTime = getTime(strAssignmentName, false,false);

                string st = convertDate(startTime);
                string et = convertDate(enforcementDeadline);
                dataGridView1.Rows[i].Cells[3].Value = st;
                dataGridView1.Rows[i].Cells[4].Value = et;
                int d = i+1;
                lblCount.Text = d.ToString();
            }
            
        }

        private string convertDate(string UTC)
        {
            try
            {
                DateTime dt = System.Management.ManagementDateTimeConverter.ToDateTime(UTC).AddHours(-1);
                return dt.ToString();
            }
            catch { return null; }
        }



        public bool CreateMaintenanceWindow(WqlConnectionManager connection,
                                    string targetCollectionID,
                                    string newMaintenanceWindowName,
                                    string newMaintenanceWindowDescription,
                                    string newMaintenanceWindowServiceWindowSchedules,
                                    bool newMaintenanceWindowIsEnabled,
                                    int newMaintenanceWindowServiceWindowType)
        {
            try
            {
                // Create an object to hold the collection settings instance (used to check whether a collection settings instance exists). 
                IResultObject collectionSettingsInstance = null;

                // Get the collection settings instance for the targetCollectionID.
                IResultObject allCollectionSettings = connection.QueryProcessor.ExecuteQuery("Select * from SMS_CollectionSettings where CollectionID='" + targetCollectionID + "'");

                // Enumerate the allCollectionSettings collection (there should be just one item) and save the instance.
                foreach (IResultObject collectionSetting in allCollectionSettings)
                {
                    collectionSettingsInstance = collectionSetting;
                }

                // If a collection settings instance does not exist for the target collection, create one.
                if (collectionSettingsInstance == null)
                {
                    collectionSettingsInstance = connection.CreateInstance("SMS_CollectionSettings");
                    collectionSettingsInstance["CollectionID"].StringValue = targetCollectionID;
                    collectionSettingsInstance.Put();
                    collectionSettingsInstance.Get();
                }

                // Create a new array list to hold the service window object.
                List<IResultObject> tempServiceWindowArray = new List<IResultObject>();

                // Create and populate a temporary SMS_ServiceWindow object with the new maintenance window values.
                IResultObject tempServiceWindowObject = connection.CreateEmbeddedObjectInstance("SMS_ServiceWindow");

                // Populate temporary SMS_ServiceWindow object with the new maintenance window values.
                tempServiceWindowObject["Name"].StringValue = newMaintenanceWindowName;
                tempServiceWindowObject["Description"].StringValue = newMaintenanceWindowDescription;
                tempServiceWindowObject["ServiceWindowSchedules"].StringValue = newMaintenanceWindowServiceWindowSchedules;
                tempServiceWindowObject["IsEnabled"].BooleanValue = newMaintenanceWindowIsEnabled;
                tempServiceWindowObject["ServiceWindowType"].IntegerValue = newMaintenanceWindowServiceWindowType;

                // Populate the local array list with the existing service window objects (from the target collection).
                tempServiceWindowArray = collectionSettingsInstance.GetArrayItems("ServiceWindows");

                // Add the newly created service window object to the local array list.
                tempServiceWindowArray.Add(tempServiceWindowObject);

                // Replace the existing service window objects from the target collection with the temporary array that includes the new service window.
                collectionSettingsInstance.SetArrayItems("ServiceWindows", tempServiceWindowArray);

                // Save the new values in the collection settings instance associated with the target collection.
                collectionSettingsInstance.Put();
                return true;
            }
            catch (SmsException ex)
            {
                warning();
                txtInfo.Invoke(new MethodInvoker(delegate { txtInfo.Text = "Failed. Error: " + ex.InnerException.Message; }));
                return false;
                //throw;
            }
        }

        public bool DeleteMaintenanceWindowfromCollection(WqlConnectionManager connection, string targetCollectionID, string serviceWindowID)
        {
            try
            {
                // Create a new array list to hold the service window objects.
                List<IResultObject> tempMaintenanceWindowArray = new List<IResultObject>();

                // Establish connection to collection settings instance associated with the target collection ID.
                IResultObject collectionSettings = connection.GetInstance(@"SMS_CollectionSettings.CollectionID='" + targetCollectionID + "'");

                // Populate the array list with the existing service window objects (from the target collection).
                tempMaintenanceWindowArray = collectionSettings.GetArrayItems("ServiceWindows");

                // Enumerate through the array list to access each maintenance window object.
                foreach (IResultObject maintenanceWindow in tempMaintenanceWindowArray)
                {
                    // If the maintenance window ID matches the one passed in to the function, delete the maintenance window.
                    if (maintenanceWindow["ServiceWindowID"].StringValue == serviceWindowID)
                    {
                        tempMaintenanceWindowArray.Remove(maintenanceWindow);
                        txtInfo.Text = "Deleted:";
                        txtInfo.Text = "Maintenance Window Name: " + maintenanceWindow["Name"].StringValue;
                        txtInfo.Text = "Maintenance Windows Service Window ID: " + maintenanceWindow["ServiceWindowID"].StringValue;
                        break;
                    }
                }

                // Replace the existing service window objects from the target collection with the temporary array that includes the new maintenance window.
                collectionSettings.SetArrayItems("ServiceWindows", tempMaintenanceWindowArray);

                // Save the new values in the collection settings instance associated with the Collection ID.
                try
                {
                    collectionSettings.Put();
                    return true;
                }
                catch(Exception ex)
                {
                    warning();
                    txtInfo.Text = ex.Message;
                    return false;
                }


            }

            catch (SmsException ex)
            {
                txtInfo.Text ="Failed. Error: " + ex.InnerException.Message;
                throw;
            }
        }

        public string getServiceWindowID(WqlConnectionManager connection, string targetCollectionID)
        {
            string serviceWindowID = "";
            try
            {
                // Create an object to hold the collection settings instance (used to check whether a collection settings instance exists). 
                IResultObject collectionSettingsInstance = null;

                // Get the collection settings instance for the targetCollectionID.
                IResultObject allCollectionSettings = connection.QueryProcessor.ExecuteQuery("Select * from SMS_CollectionSettings where CollectionID='" + targetCollectionID + "'");

                // Enumerate the allCollectionSettings collection (there should be just one item) and save the instance.
                foreach (IResultObject collectionSetting in allCollectionSettings)
                {
                    collectionSettingsInstance = collectionSetting;
                }

                // If a collection settings instance, output message that there are no maintenance windows.
                if (collectionSettingsInstance == null)
                {
                    warning();
                    txtInfo.Text ="There are no maintenance windows for collection: " + targetCollectionID;
                }
                else
                {
                    // Create a new array list to hold the service window objects.
                    List<IResultObject> maintenanceWindowArray = new List<IResultObject>();

                    // Establish connection to collection settings instance associated with the Collection ID.
                    IResultObject collectionSettings = connection.GetInstance(@"SMS_CollectionSettings.CollectionID='" + targetCollectionID + "'");

                    // Populate the array list with the existing service window objects (from the target collection).
                    maintenanceWindowArray = collectionSettings.GetArrayItems("ServiceWindows");

                    // Enumerate through the array list to access each maintenance window object and output specific properties for each object.

                    foreach (IResultObject maintenanceWindow in maintenanceWindowArray)
                    {
                        serviceWindowID = maintenanceWindow["ServiceWindowID"].StringValue;
                    };
                }

                return serviceWindowID;
            }
            catch (SmsException ex)
            {
                txtInfo.Text ="Failed. Error: " + ex.InnerException.Message;
                throw;
            }
        }



        public string CreateDailyRecurringScheduleToken(WqlConnectionManager connection,
                                              int hourDuration,
                                              int daySpan,
                                              string startTime,
                                              bool isGmt)
        {
            try
            {
                // Create a new recurring interval schedule object.
                // Note: There are several types of schedule classes available, each defines a different type of schedule.
                IResultObject recurInterval = connection.CreateEmbeddedObjectInstance("SMS_ST_NonRecurring");

                // Populate the schedule properties.
                recurInterval["DayDuration"].IntegerValue = 0;
                recurInterval["HourDuration"].IntegerValue = hourDuration;
                recurInterval["MinuteDuration"].IntegerValue = 0;
                recurInterval["StartTime"].StringValue = startTime;
                recurInterval["IsGMT"].BooleanValue = isGmt;

                // Creating array to use as a parameters for the WriteToString method.
                List<IResultObject> scheduleTokens = new List<IResultObject>();
                scheduleTokens.Add(recurInterval);

                // Creating dictionary object to pass parameters to the WriteToString method.
                Dictionary<string, object> inParams = new Dictionary<string, object>();
                inParams["TokenData"] = scheduleTokens;

                // Initialize the outParams object.
                IResultObject outParams = null;

                // Call WriteToString method to decode the schedule token.
                outParams = connection.ExecuteMethod("SMS_ScheduleMethods", "WriteToString", inParams);

                // Output schedule token as an interval string.
                return outParams["StringData"].StringValue;
                
            }
            catch (SmsException ex)
            {
                warning();
                txtInfo.Text = "Failed. Error: " + ex.InnerException.Message;
                return "ERROR";
            }
        }

        private void returnRegValues()
        {

            RegistryKey regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Bondytech\\SoftwareUpdateTool");

            foreach (string subKeyName in regKey.GetSubKeyNames())
            {
                using (RegistryKey tempKey = regKey.OpenSubKey(subKeyName))
                {
                    txtInfo.Text = (tempKey.ValueCount.ToString() + "\n" + tempKey.Name);
                    foreach (string valueName in tempKey.GetValueNames())
                    {
                        txtInfo.Text = valueName + "\n" + tempKey.GetValue(valueName).ToString();
                    }
                }
            }

        }

        private void button1_Click_1(object sender, EventArgs e)
        {
            WqlConnectionManager connection = WMIConn();

            string serverFQDN = cboSiteServer.Text + "." + txtDomain.Text;

            IResultObject siteCode = connection.QueryProcessor.ExecuteQuery("SELECT * FROM SMS_Site Where ServerName = '" + serverFQDN + "'");

            //MessageBox.Show(siteCode["SiteCode"].ToString());
            // Enumerate the allCollectionSettings collection (there should be just one item) and save the instance.
            foreach (IResultObject code in siteCode)
            {
                MessageBox.Show(code["SiteCode"].StringValue);
            }

        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmAbout fa = new frmAbout(this);
            fa.StartPosition = FormStartPosition.CenterParent;
            fa.ShowDialog(this);
        }

        private void helpToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            frmHelp fh = new frmHelp(this);
            fh.StartPosition = FormStartPosition.CenterParent;
            fh.ShowDialog(this);
        }

        private void button1_Click_2(object sender, EventArgs e)
        {
            DateTime date = DateTime.Now;
            int patchMonth = 0;
            lstPatchMonth.Invoke(new MethodInvoker(delegate { patchMonth = Convert.ToDateTime(lstPatchMonth.SelectedItem.ToString() + " 01, 1900").Month; }));
            int yr = date.Year;
            int mo = date.Month;
            if (patchMonth > mo) yr--;
            if (mo != patchMonth) mo = patchMonth;
            DateTime _date = new DateTime(yr, mo, 12);
            DateTime PatchSunday = _date.AddDays(2 - Convert.ToInt32(_date.DayOfWeek) + 5);
            MessageBox.Show(PatchSunday.Day.ToString());
        }

        private void dualEnvironmentToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (dualEnvironmentToolStripMenuItem1.Checked)
                setRegValue("DualEnvironment", "1");
            else
                setRegValue("DualEnvironment", "0");
        }

        private void addMaintenanceWindowsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if(addMaintenanceWindowsToolStripMenuItem.Checked)
                setRegValue("ServiceWindow", "1");
            else
                setRegValue("ServiceWindow", "0");

        }



        private bool clearMaintenanceWindows(WqlConnectionManager connection)
        {
            try
            {
                string Query1 = "SELECT * FROM SMS_Collection WHERE Name Like '%Software Updates - %'";
                string CollectionID = "";
                string ServiceWindowID = "";
                string collectionName = "";
                bool completed = false;
                IResultObject SoftwareUpdateCollections = connection.QueryProcessor.ExecuteQuery(Query1);

                foreach (IResultObject collection in SoftwareUpdateCollections)
                {
                    CollectionID = collection["CollectionID"].StringValue;
                    collectionName = collection["Name"].StringValue;
                    ServiceWindowID = getServiceWindowID(connection, CollectionID);
                    completed = DeleteMaintenanceWindowfromCollection(connection, CollectionID, ServiceWindowID);
                }

                green();
                txtInfo.Text = "Removed maintenance window corresponding to collection: " + collectionName;
                return completed;
            }
            catch (SmsException ex)
            {
                warning();
                txtInfo.Text = "Failed to delete all maintenance windows. Error: " + ex.Message;
                Log("Failed to delete all maintenance windows. Error: " + ex.Message);
                return false;
            }
        }

        private void clearAllMaintenanceWindowsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            clearMaintenanceWindows(WMIConn());
        }


    }
 
}
